@@ -510,21 +510,21 @@ namespace FSpot {
 		{
 			Gtk.Menu order_menu = new Gtk.Menu();
 			
-			GtkUtil.MakeCheckMenuItem (order_menu, Mono.Posix.Catalog.GetString ("Arrange by _Month"),
+			GtkUtil.MakeCheckMenuItem (order_menu, Catalog.GetString ("Arrange by _Month"),
 					      MainWindow.Toplevel.HandleArrangeByTime, true, (adaptor is TimeAdaptor), true);
 			
-			GtkUtil.MakeCheckMenuItem (order_menu, Mono.Posix.Catalog.GetString ("Arrange by _Folder"),
+			GtkUtil.MakeCheckMenuItem (order_menu, Catalog.GetString ("Arrange by _Folder"),
 					      MainWindow.Toplevel.HandleArrangeByDirectory, true, (adaptor is DirectoryAdaptor), true);
 
 			GtkUtil.MakeMenuSeparator (order_menu);
 
-			GtkUtil.MakeCheckMenuItem (order_menu, Mono.Posix.Catalog.GetString ("_Reverse Order"),
+			GtkUtil.MakeCheckMenuItem (order_menu, Catalog.GetString ("_Reverse Order"),
 					      MainWindow.Toplevel.HandleReverseOrder, true, adaptor.OrderAscending, false);
 
 			if (adaptor is TimeAdaptor && adaptor.Query.Range != null) {
 				GtkUtil.MakeMenuSeparator (order_menu);
 
-				GtkUtil.MakeMenuItem (order_menu, Mono.Posix.Catalog.GetString ("_Clear Date Range"), 
+				GtkUtil.MakeMenuItem (order_menu, Catalog.GetString ("_Clear Date Range"), 
 						MainWindow.Toplevel.HandleClearDateRange);
 			}
 			
@@ -813,10 +813,10 @@ namespace FSpot {
 						   bounds.Width, handle_height, Orientation.Horizontal);
 				
 				Style.PaintShadow (selector.Style, selector.GdkWindow, State, ShadowType.Out, 
-						   area, selector, "glass", bounds.X, bounds.Y, bounds.Width, bounds.Height);
+						   area, selector, null, bounds.X, bounds.Y, bounds.Width, bounds.Height);
 				
 				Style.PaintShadow (selector.Style, selector.GdkWindow, State, ShadowType.In, 
-						   area, selector, "glass", inner.X, inner.Y, inner.Width, inner.Height);
+						   area, selector, null, inner.X, inner.Y, inner.Width, inner.Height);
 				
 			}
 			
@@ -884,10 +884,10 @@ namespace FSpot {
 						    selector, "IconView:Selector", bottom_points, true);
 #else
 				Style.PaintBox (selector.Style, selector.GdkWindow, State, ShadowType.Out, area,
-						selector, "limit", top.X, top.Y, top.Width, top.Height);
+						selector, null, top.X, top.Y, top.Width, top.Height);
 
 				Style.PaintBox (selector.Style, selector.GdkWindow, State, ShadowType.Out, area,
-						selector, "limit", bottom.X, bottom.Y, bottom.Width, bottom.Height);
+						selector, null, bottom.X, bottom.Y, bottom.Width, bottom.Height);
 #endif
 			}
 
