@@ -3,6 +3,7 @@ using System.Collections;
 using System.Runtime.InteropServices;
 using System;
 using System.IO;
+using FSpot;
 
 /**
   1        2       3      4         5            6           7          8
@@ -162,14 +163,12 @@ class PixbufUtils {
 		
 		public Pixbuf LoadFromFile (string path)
 		{
-			FileStream fs = null;
-			
 			try {
 				orientation = GetOrientation (path);
-				using (fs = File.OpenRead (path)) {
+				using (FileStream fs = File.OpenRead (path)) {
 					return Load (fs, orientation);
 				}
-			} catch (Exception e) {
+			} catch (Exception) {
 				System.Console.WriteLine ("Error loading photo {0}", path);
 				return null;
 			} 
@@ -295,6 +294,18 @@ class PixbufUtils {
 		stream.Write (data, 0, data.Length);
 	}
 
+	static string [] NullTerminateArray (string [] options)
+	{
+		string [] terminated_options = options;
+
+		if (options != null && options [ options.Length - 1 ] != null) {
+			terminated_options = new string [options.Length + 1];
+			Array.Copy (options, terminated_options, options.Length);
+		}
+		
+		return terminated_options;
+	}
+
 	[DllImport("libgdk_pixbuf-2.0-0.dll")]
 	static extern bool gdk_pixbuf_save_to_bufferv (IntPtr raw, out IntPtr data, out IntPtr length, 
 						       string type, 
@@ -306,31 +317,14 @@ class PixbufUtils {
 		IntPtr error = IntPtr.Zero;
 		IntPtr data;
 		IntPtr length;
-		string [] terminated_options = options;
-		string [] terminated_values = values;
-
-		if (options != null && options [ options.Length - 1 ] != null) {
-			terminated_options = new string [options.Length + 1];
-			Array.Copy (options, terminated_options, options.Length);
-		}
-
-		if (values != null && values [ values.Length - 1] != null) {
-			terminated_values = new string [values.Length + 1];
-			Array.Copy (values, terminated_values, values.Length);
-		}
-		
-		if (terminated_options != null)
-			System.Console.WriteLine ("options = {0}, values = {1}", terminated_options [0], terminated_values [0]);
-		else
-			Console.WriteLine ("no options");
 
 		bool success = gdk_pixbuf_save_to_bufferv (pixbuf.Handle, 
-							     out data, 
-							     out length, 
-							     type,
-							     terminated_options,
-							     terminated_values,
-							     out error);
+							   out data, 
+							   out length, 
+							   type,
+							   NullTerminateArray (options),
+							   NullTerminateArray (values),
+							   out error);
 		
 		if (error != IntPtr.Zero) 
 			throw new GLib.GException (error);
@@ -341,6 +335,8 @@ class PixbufUtils {
 		byte [] content = new byte [(int)length];
 		Marshal.Copy (data, content, 0, (int)length);
 
+		GLib.Marshaller.Free (data);
+
 		return content;
 	}
 	
@@ -406,7 +402,7 @@ class PixbufUtils {
 	static public Pixbuf LoadFromAssembly (string resource)
 	{
 		try {
-			return new Pixbuf (System.Reflection.Assembly.GetCallingAssembly (), resource);
+			return new Pixbuf (System.Reflection.Assembly.GetEntryAssembly (), resource);
 		} catch {
 			return null;
 		}
@@ -418,7 +414,7 @@ class PixbufUtils {
 	public static void SaveAtomic (Gdk.Pixbuf src, string filename, string type, string [] keys, string [] values)
 	{
 			string tmpname = filename + ".tmp";
-			src.Savev (tmpname, type, keys, values);
+			src.Savev (tmpname, type, NullTerminateArray (keys), NullTerminateArray (values));
 			if (rename (tmpname, filename) < 0)
 				throw new Exception ("Error renaming file");
 	}
@@ -573,17 +569,62 @@ class PixbufUtils {
 		return ret;
 	}	
 	
+	[DllImport ("libfspot")]
+	static extern IntPtr f_pixbuf_blur (IntPtr src, double radius);
+
+	public static Pixbuf Blur (Pixbuf src, double radius)
+	{
+		IntPtr raw_ret = f_pixbuf_blur (src.Handle, radius);
+ 		Gdk.Pixbuf ret = (Gdk.Pixbuf) GLib.Object.GetObject(raw_ret, true);
+		return ret;
+	}	
+
+#if OLDREDEYE
 	[DllImport ("libfspot")]
 	static extern void f_pixbuf_remove_redeye (IntPtr src);
-	
+
 	public static Gdk.Pixbuf RemoveRedeye (Gdk.Pixbuf src, Gdk.Rectangle area)
+#else
+	public unsafe static Gdk.Pixbuf RemoveRedeye (Gdk.Pixbuf src, Gdk.Rectangle area)
+	{
+		return RemoveRedeye (src, area, -15);
+	}
+
+	public unsafe static Gdk.Pixbuf RemoveRedeye (Gdk.Pixbuf src, Gdk.Rectangle area, int threshold)
+	//threshold, factors and comparisons borrowed from the gimp plugin 'redeye.c' by Robert Merkel
+#endif
 	{
 		Gdk.Pixbuf copy = src.Copy ();
 		Gdk.Pixbuf selection = new Gdk.Pixbuf (copy, area.X, area.Y, area.Width, area.Height);
-
+#if OLREDEYE
 		f_pixbuf_remove_redeye (selection.Handle);
 		selection.Dispose ();
+#else
+		byte *spix = (byte *)selection.Pixels;
+		int h = selection.Height;
+		int w = selection.Width;
+		int channels = src.NChannels;
 
+		double RED_FACTOR = 0.5133333;
+		double GREEN_FACTOR = 1;
+		double BLUE_FACTOR = 0.1933333;
+
+		for (int j = 0; j < h; j++) {
+			byte *s = spix;
+			for (int i = 0; i < w; i++) {
+				int adjusted_red = (int)(s[0] * RED_FACTOR);
+				int adjusted_green = (int)(s[1] * GREEN_FACTOR);
+				int adjusted_blue = (int)(s[2] * BLUE_FACTOR);
+
+				if (adjusted_red >= adjusted_green - threshold
+				    && adjusted_red >= adjusted_blue - threshold)
+					s[0] = (byte)(((double)(adjusted_green + adjusted_blue)) / (2.0 * RED_FACTOR));
+				s += channels;
+			}
+			spix += selection.Rowstride;
+		}
+
+#endif
 		return copy;
 	}
 
@@ -645,19 +686,71 @@ class PixbufUtils {
 		
 	}
 
+	public static unsafe bool IsGray (Gdk.Pixbuf pixbuf, int max_difference)
+	{
+		int chan = pixbuf.NChannels;
+
+		byte *pix = (byte *)pixbuf.Pixels;
+		int h = pixbuf.Height;
+		int w = pixbuf.Width;
+		int stride = pixbuf.Rowstride;
+
+		for (int j = 0; j < h; j++) {
+			byte *p = pix;
+			for (int i = 0; i < w; i++) {
+				if (Math.Abs (p[0] - p[1]) > max_difference || Math.Abs (p[0] - p [2]) > max_difference) {
+					goto Found;
+				}
+				p += chan;
+			}
+			pix += stride;
+		}
+
+		return true;
+
+	Found:
+		return false;
+	}
+
+	public static unsafe void ReplaceColor (Gdk.Pixbuf src, Gdk.Pixbuf dest)
+	{
+		if (src.HasAlpha || !dest.HasAlpha || (src.Width != dest.Width) || (src.Height != dest.Height))
+			throw new ApplicationException ("invalid pixbufs");
+
+		byte *dpix = (byte *)dest.Pixels;
+		byte *spix = (byte *)src.Pixels;
+		int h = src.Height;
+		int w = src.Width;
+		for (int j = 0; j < h; j++) {
+			byte *d = dpix;
+			byte *s = spix;
+			for (int i = 0; i < w; i++) {
+				d[0] = s[0];
+				d[1] = s[1];
+				d[2] = s[2];
+				d += 4;
+				s += 3;
+			}
+			dpix += dest.Rowstride;
+			spix += src.Rowstride;
+		}
+	}
+
 	public static Gdk.Pixbuf GetThumbnail (Exif.ExifData data)
 	{
 		byte [] thumb_data = data.Data;
 		if (thumb_data.Length > 0) {
 			PixbufOrientation orientation = GetOrientation (data);
-			MemoryStream mem = new MemoryStream (thumb_data);
-			Gdk.Pixbuf thumb = new Gdk.Pixbuf (mem);
-			Gdk.Pixbuf rotated = PixbufUtils.TransformOrientation (thumb, orientation);
 			
-			if (rotated != thumb)
-				thumb.Dispose ();
-			
-			return rotated;
+			using (MemoryStream mem = new MemoryStream (thumb_data)) {
+				Gdk.Pixbuf thumb = new Gdk.Pixbuf (mem);
+
+				Gdk.Pixbuf rotated = PixbufUtils.TransformOrientation (thumb, orientation);
+				
+				if (rotated != thumb)
+					thumb.Dispose ();
+				return rotated;
+			}			
 		}
 		return null;
 	}
@@ -801,26 +894,18 @@ class PixbufUtils {
 						 trect.Width, trect.Height);
 
 		Gdk.Rectangle subarea;
-		while (rect.Y < args.Y + args.Height) {
-			while (rect.X < args.X + args.Width) {
-				rect.Intersect (args, out subarea);
-				Gdk.Rectangle trans = TransformOrientation (src, subarea, orientation);
+		BlockProcessor proc = new BlockProcessor (args, 256);
+		while (proc.Step (out subarea)) {
+			Gdk.Rectangle trans = TransformOrientation (src, subarea, orientation);
+			Gdk.Pixbuf ssub = new Gdk.Pixbuf (src, subarea.X, subarea.Y,
+							  subarea.Width, subarea.Height);
 
-			        Gdk.Pixbuf ssub = new Gdk.Pixbuf (src, subarea.X, subarea.Y,
-								  subarea.Width, subarea.Height);
+			Gdk.Pixbuf tsub = new Gdk.Pixbuf (tmp, 0, 0, trans.Width, trans.Height);
+			CopyWithOrientation (ssub, tsub, orientation);
 
-				Gdk.Pixbuf tsub = new Gdk.Pixbuf (tmp, 0, 0, trans.Width, trans.Height);
-
-				CopyWithOrientation (ssub, tsub, orientation);
-
-				tsub.CopyArea (0, 0, trans.Width, trans.Height, dest, trans.X, trans.Y);
-				
-				ssub.Dispose ();
-				tsub.Dispose ();
-				rect.X += rect.Width;
-			}
-			rect.X = args.X;
-			rect.Y += rect.Height;
+			tsub.CopyArea (0, 0, trans.Width, trans.Height, dest, trans.X, trans.Y);
+			ssub.Dispose ();
+			tsub.Dispose ();
 		}
 
 		tmp.Dispose ();
